@@ -25,6 +25,7 @@
 
 #ifdef HAVE_LIBGPHOTO
 
+#include <unistd.h>
 #include <string.h>
 #include <strings.h>
 #include <time.h>
@@ -1953,6 +1954,7 @@ dlg_photo_importer (GthBrowser *browser)
 	GdkPixbuf  *mute_pixbuf;
 	char       *default_path;
 	char       *default_film_name;
+	char       *default_uri;
 
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
@@ -2052,10 +2054,12 @@ dlg_photo_importer (GthBrowser *browser)
 	default_path = eel_gconf_get_path (PREF_PHOTO_IMPORT_DESTINATION, NULL);
 	if ((default_path == NULL) || (*default_path == 0))
 		default_path = xdg_user_dir_lookup ("PICTURES");
-	
+	default_uri = add_scheme_if_absent (default_path);
+		
 	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (data->destination_filechooserbutton),
-					         default_path);
+					         default_uri);
 	g_free (default_path);
+	g_free (default_uri);
 
 	default_film_name = eel_gconf_get_path (PREF_PHOTO_IMPORT_FILM, "");
 	_gtk_entry_set_filename_text (GTK_ENTRY (data->film_entry), default_film_name);
