@@ -141,6 +141,9 @@ typedef struct {
 
 	GHashTable     *folders_comment;
 	GHashTable     *hidden_files;
+	GHashTable     *visited_dirs;
+	
+	gboolean        fast_file_type;
 } DialogData;
 
 
@@ -185,13 +188,18 @@ free_search_results_data (DialogData *data)
 		file_data_list_free (data->files);
 		data->files = NULL;
 	}
-
+	
 	if (data->dirs) {
 		path_list_free (data->dirs);
 		data->dirs = NULL;
 	}
-
+	
 	g_hash_table_foreach_remove (data->folders_comment, remove_folder_comment_cb, NULL);
+	
+	if (data->visited_dirs != NULL) {
+		g_hash_table_destroy (data->visited_dirs);
+		data->visited_dirs = NULL;
+	}
 }
 
 
@@ -320,6 +328,7 @@ static void
 destroy_progress_cb (GtkWidget *widget,
 		     DialogData *data)
 {
+	gth_file_list_stop (data->file_list);
 	cancel_progress_dlg_cb (widget, data);
 	gtk_widget_destroy (data->dialog);
 }
@@ -448,7 +457,6 @@ static void
 cancel_progress_dlg_cb (GtkWidget  *widget,
 			DialogData *data)
 {
-	gth_file_list_stop (data->file_list);
 	if (data->handle == NULL)
 		return;
 	gnome_vfs_async_cancel (data->handle);
@@ -550,6 +558,8 @@ dlg_search_ui (GthBrowser *browser,
 	data->catalog_path = catalog_path;
 	data->folders_comment = g_hash_table_new (g_str_hash, g_str_equal);
 	data->hidden_files = NULL;
+	data->visited_dirs = NULL;
+	data->fast_file_type = eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE);
 
 	/* Get the widgets. */
 
@@ -1003,7 +1013,8 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 	for (node = list; node != NULL; node = node->next) {
 		GnomeVFSURI *full_uri = NULL;
 		char        *str_uri;
-		char	    *unesc_uri;
+		char        *real_uri;
+		char	    *unesc_uri;  
 
 		info = node->data;
 
@@ -1015,8 +1026,13 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
 			unesc_uri = gnome_vfs_unescape_string (str_uri, "");
 
-			if (file_respects_search_criteria (data, unesc_uri))
-				files = g_list_prepend (files, file_data_new (str_uri, info));
+			if (file_respects_search_criteria (data, unesc_uri)) {
+				FileData *file;
+				
+				file = file_data_new (str_uri, info);
+				file_data_update_mime_type (file, data->fast_file_type);				
+				files = g_list_prepend (files, file);
+			}
 			else
 				g_free (str_uri);
 
@@ -1029,7 +1045,15 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			if (g_hash_table_lookup (data->hidden_files, info->name) != NULL)
 				break;
 			full_uri = gnome_vfs_uri_append_path (data->uri, info->name);
-			data->dirs = g_list_prepend (data->dirs, gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE));
+			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
+			if (resolve_all_symlinks (str_uri, &real_uri) == GNOME_VFS_OK) {
+				if (g_hash_table_lookup (data->visited_dirs, real_uri) == NULL) { 
+					data->dirs = g_list_prepend (data->dirs, g_strdup (real_uri));
+					g_hash_table_insert (data->visited_dirs, g_strdup (real_uri), GINT_TO_POINTER (1));
+				}
+				g_free (real_uri);
+			}
+			g_free (str_uri);
 			break;
 
 		default:
@@ -1043,9 +1067,19 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 	if (files != NULL)
 		add_file_list (data, files);
 
-	if (result == GNOME_VFS_ERROR_EOF) {
+	if (result != GNOME_VFS_OK) {
 		gboolean good_dir_to_search_into = TRUE;
 
+		if (result != GNOME_VFS_ERROR_EOF) {
+			char *path;
+
+			path = gnome_vfs_uri_to_string (data->uri,
+							GNOME_VFS_URI_HIDE_NONE);
+			/*g_warning ("Cannot load directory \"%s\": %s\n", path,
+				   gnome_vfs_result_to_string (result));*/
+			g_free (path);
+		}
+
 		if (! data->search_data->recursive) {
 			search_finished (data);
 			return;
@@ -1071,17 +1105,6 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			g_free (dir);
 		} while (! good_dir_to_search_into);
 	} 
-	else if (result != GNOME_VFS_OK) {
-		char *path;
-
-		path = gnome_vfs_uri_to_string (data->uri,
-						GNOME_VFS_URI_HIDE_NONE);
-		g_warning ("Cannot load directory \"%s\": %s\n", path,
-			   gnome_vfs_result_to_string (result));
-		g_free (path);
-
-		search_finished (data);
-	}
 }
 
 
@@ -1089,18 +1112,30 @@ static void
 search_dir_async (DialogData *data,
 		  char       *dir)
 {
+	char *uri, *real_uri;
+	
 	_gtk_entry_set_filename_text (GTK_ENTRY (data->p_current_dir_entry), dir);
 
 	/**/
 
 	if (data->uri != NULL)
 		gnome_vfs_uri_unref (data->uri);
-	data->uri = new_uri_from_path (dir);
+	
+	uri = add_scheme_if_absent (dir);
+	if (! resolve_all_symlinks (uri, &real_uri) == GNOME_VFS_OK) {
+		g_free (uri);
+		return;
+	}
+	g_free (uri);
+		 
+	g_hash_table_insert (data->visited_dirs, g_strdup (real_uri), GINT_TO_POINTER (1));
+	data->uri = new_uri_from_path (real_uri);
+	g_free (real_uri);
 
 	if (data->hidden_files != NULL)
 		g_hash_table_destroy (data->hidden_files);
 	data->hidden_files = read_dot_hidden_file (dir);
-
+				      
 	gnome_vfs_async_load_directory_uri (
 		& (data->handle),
 		data->uri,
@@ -1127,8 +1162,11 @@ search_images_async (DialogData *data)
 	SearchData *search_data = data->search_data;
 
 	free_search_results_data (data);
+	data->visited_dirs = g_hash_table_new_full (g_str_hash,
+					      	    g_str_equal,
+					            (GDestroyNotify) g_free,
+					            NULL);
 	gth_file_list_set_list (data->file_list, NULL, pref_get_arrange_type (), pref_get_sort_order ());
-
 	search_dir_async (data, search_data->start_from);
 }
 
